@@ -99,6 +99,7 @@ Facebook::OpenGraph::Response - Response object for Facebook::OpenGraph.
       content     => $response_content,
       req_headers => $req_headers,
       req_content => $req_content,
+      json        => JSON->new->utf8,
   });
 
 =head1 DESCRIPTION
@@ -141,6 +142,10 @@ Stringified request headers
 
 Request content
 
+=item * json
+
+JSON object
+
 =back
 
 =head2 Instance Methods
